/*  Copyright (C) 2015 Zoltán Nagy
 *  Created by Zoltán Nagy on 1/9/15.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */
#include "bits/mc-basic-user.h"


namespace duct {
  
  thread_local std::function<void(basic_user_base *)> basic_user_base::initializer = nullptr;

  
  fs::path basic_user_base::_M_getfname(bool mkd) const
  {
    fs::path fname{_M_dname};
    
    /*   directory of the output file   */
    fname /= _M_pname;
    fname /= _M_uname;
    
    /*   create the directory if it is necessary   */
    if(mkd) create_directories(fname);
    
    /*   the full filename of the output file   */
    fname /= _M_rname+".dat";
        
    return fname;
  }
  
  
  
  void basic_user_base::operations_at_the_end_of_event()
  {
    this -> basic_user_end_of_event();
    
    /*   check the time   */
    auto now = std::chrono::system_clock::now();
    std::chrono::duration<double> elapsed = now - _M_timer;
    
    if(elapsed >= _M_save_period || user_base::is_done())
    {
      std::fstream ofile(_M_getfname(true).c_str(), std::ios::out);
      
      /*   number of events   */
      unsigned long long int nevent = user_base::number_of_events();
      
      /*   first byte of the file tells if the stream is txt or binary   */
      ::duct::write(ofile, _M_uname, std::string{c_name_of_collect_and_print()}, nevent);
      this -> write(ofile);
      
      ofile.flush();
      ofile.close();
      
      std::string dir = _M_dname.string();
      auto pos = dir.find_last_of('/');
      pos = (pos == std::string::npos ? 0 : pos+1);
      
      /*   reset the timer    */
      _M_timer = std::chrono::system_clock::now();
    }
  }
  

  void basic_user_base::finish()
  {
    std::cout<<"\nAnalyser \'"<<_M_uname<<"\' has finished its job and saved the results into file \'"<<_M_getfname(false)<<"\'.\n";
    std::cout<<"This file has binary format and contains raw data, so you should produce the (human readable) results by using function\n";
    std::cout<<"\'__basic_user_make_result<...>::collect_and_print(const char *, const char **)\'\n";
    std::cout<<"or a program based on this function, like \'deductor --result ...\'.\n"<<std::endl;
  }

}

