/*  Copyright (C) 2010-2013 Zoltán Nagy
 *  Created by Zoltan Nagy on 10/5/10.
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef __duct_mapping_h__
#define __duct_mapping_h__ 1


/*   std includes   */
#include <functional>

/*   project headers   */
#include <bits/duct-tags.h>
#include <bits/duct-event.h>



namespace duct {
  
  /*  Splitting variables
   *  This class is a collection of variables for reconstracting
   *  the event after a splitting.
   */
  template<class _Tag>
  struct splitting_variables;
  
  
  /*   Set of splitting variables in a LO shower   */
  template<>
  struct splitting_variables<shower<lo>>
  {
    /*  label of the emitter and the possible spectator   */
    int emitter, spectator;
    
    /*  flavor splitting variable   */
    int fl, fm;
    
    /*  momentum splitting variables   */
    double y, z, phi;
    
    /*  color splitting variables   */
    int sigma, sigmaP;
    
    /*   this is the shower scale in GeV^2 (ini: v*Q2 or fin: v*pQ)   */
    double scale;
  };

  
  
  namespace __details {
    
    /*  This function does the mapping in the color, flavor and momentum space.
     *  This is not really a public function, it is used only in the shower evolution.
     *  Once C++ provides support for modules, it will be private member of the namespace.
     */
    template<class _Tag> std::function<lorentzvector<double>(const lorentzvector<double>&)>
    shower_mapping(event<_Tag>&, const event<_Tag>&, const splitting_variables<shower<lo>>&);
    
    template<class _Tag> std::function<lorentzvector<double>(const lorentzvector<double>&)>
    shower_mapping_no_color(event<_Tag>&, const event<_Tag>&, const splitting_variables<shower<lo>>&);
    
    template<class _Tag>
    void shower_color_mapping(event<_Tag>&, const event<_Tag>&, const splitting_variables<shower<lo>>&);
    
  }
  
}

#endif

